@@ -36,7 +36,7 @@ from collective.releaser.base import savewd
 join = os.path.join
 
 extract_path = re.compile(r'^((?:htt(?:p|ps)|svn|file)://)(.*?)/?$')
-is_svn = re.compile(r'^((?:htt(?:p|ps)|svn|file)://)(.*?)/?$') 
+is_svn = re.compile(r'^((?:htt(?:p|ps)|svn|file)://)(.*?)/?$')
 
 def _log(msg):
     print msg
@@ -50,6 +50,8 @@ def _extract_url(url):
 @savewd
 def make_release(version=None):
     """this is called when a release is made over a buildout"""
+    if not base.has_svn_support():
+        raise base.ReleaseError('You need Subversion to create a release tag')
 
     # first, let's check we are in the buildout folder
     dir = os.getcwd()
@@ -84,7 +86,7 @@ def make_release(version=None):
     # let's get it to add a version.txt file in the buildout root
     rep = mkdtemp()
     try:
-        base.svn_checkout(release, rep)  
+        base.svn_checkout(release, rep)
         os.chdir(rep)
         version_file = join(rep, 'version.txt')
         open(version_file, 'w').write(version)
@@ -94,7 +96,7 @@ def make_release(version=None):
         base.svn_commit(msg)
     finally:
         shutil.rmtree(rep, ignore_errors=True)
-        
+
 def parse_url(url):
     """return base_url, cfg::
 
@@ -112,19 +114,6 @@ def parse_url(url):
     filename = url.pop()
     return '/'.join(url), filename
 
-def check_python(valid_version=(2, 4)):
-    """raises if not the right python"""
-    # need to but requirement somewhere un buildout
-    version = sys.version_info[0:len(valid_version)]
-    if version != valid_version:
-        version = '.'.join([str(step) for step in version])
-        valid_version = '.'.join([str(step) for step in valid_version])
-        msg = 'Found Python %s, need %s' % (version, valid_version)
-        raise base.ReleaseError(msg)
-    else:
-        return '.'.join([str(v) for v in version])
-
-
 def diff_releases(old=None, new=None, result=None):
     """takes two tarballs, and generates a diff one"""
     if old is None:
@@ -150,7 +139,7 @@ def diff_releases(old=None, new=None, result=None):
     else:
         working_dir, result = os.path.split(result)
         working_dir = os.path.realpath(working_dir)
-    
+
     old_files = {}
     for f in old_tarball.getmembers():
         old_files[f.name] = f
@@ -203,13 +192,13 @@ def archive_contents(archive, location, exclude=None, source=True):
             if skip:
                 continue
             # archiving empty dirs too
-            for dir_ in dirs:   
+            for dir_ in dirs:
                 if '.svn' in dir_.split(os.path.sep):
                     continue
                 fullpath = join(root, dir_)
                 if os.listdir(fullpath) == []:
                     arcname = fullpath.replace(location, '.')
-                    tar.add(fullpath, arcname, False) 
+                    tar.add(fullpath, arcname, False)
             for filename in filenames:
                 if filename in ('.installed.cfg', '.Python'):
                     continue
@@ -302,20 +291,20 @@ def _make_python(location='.'):
                     wanted = 'python'
                 if os.path.split(installed)[-1] != wanted:
                     dirname = os.path.dirname(installed)
-                    shutil.copyfile(installed, join(dirname, 
-                                                            wanted)) 
-                    #make it executable   
+                    shutil.copyfile(installed, join(dirname,
+                                                            wanted))
+                    #make it executable
                     try:
                         os.chmod(join(dirname, wanted), 755)
                     except OSError:
-                        pass # win32 error, or rights error    
+                        pass # win32 error, or rights error
             main()
-            #XXX: this is a hack: the after_install is never called in 
+            #XXX: this is a hack: the after_install is never called in
             # the main(). This should be fixed otherway
             after_install(None, location)
         except OSError:
             # file must exist
-            pass 
+            pass
     finally:
         sys.argv = old
     if sys.platform == 'win32':
@@ -326,30 +315,31 @@ def _make_python(location='.'):
 @savewd
 def deploy_release(path=None, target=None, archiving='full'):
     """deploy a release in-place"""
-    _log('Checking python version ...')
-    _log('%s ok.' % check_python())
     if path is None:
         if len(sys.argv) < 2:
             print 'usage : project_deploy [http://to/your/buildout/]config.cfg'
             sys.exit(1)
         path = sys.argv[1]
-    url, filename = parse_url(path)
-    if url == '':
+
+    location, filename = parse_url(path)
+    folder = location.split('/')[-1]
+
+    if os.path.isfile(path):
         _log("Using local configuration file %s" % filename)
         target, filename = os.path.split(path)
-        release_name = filename.split('.')[0]  
+        release_name = filename.split('.')[0]
     else:
-        _log("Using network file %s with url %s" % (filename, url))
-        folder = url.split('/')[-1]
+        _log("Using network file %s with url %s" % (filename, location))
         release_name = '%s-%s' % (folder, filename.split('.')[0])
-        if not target:
-            target = release_name
 
-        if os.path.isfile(path) or os.path.isdir(path):
-            target = url
+    if not target:
+        target = release_name
 
-        if not os.path.isdir(target):
-            os.mkdir(target)
+    if os.path.isfile(path) or os.path.isdir(path):
+        target = location
+
+    if not os.path.isdir(target):
+        os.mkdir(target)
 
     root = os.path.realpath(target)
     os.chdir(root)
@@ -357,28 +347,28 @@ def deploy_release(path=None, target=None, archiving='full'):
     # XXX at this time we are rebuilding everything
     # a pre-built release will have these subdirectories
     # included to avoid compiling again
-    exclude = [join(root, subfolder) for subfolder in 
+    exclude = [join(root, subfolder) for subfolder in
                ['parts', 'var', 'develop-eggs', 'bin', 'lib',
                 'Lib', 'Scripts', join('downloads', 'dist')]]
-    
-    
+
+
     # changes all paths
     if not os.path.isfile(path) and not os.path.isdir(path):
         # let's get the buildout
         if is_svn.search(path) is None:
-            raise base.ReleaseError('%s is not a svn url' % path) 
-        _log(base.system('svn co %s .' % url))
-        
+            raise base.ReleaseError('%s is not a svn url' % path)
+        _log(base.system('svn co %s .' % location))
+
     _log('Using local directory %s with %s' % (target, filename))
 
     # let's generate a virtualenv python
     # to make sure we don't loose eggs
     _make_python()
-       
+
     # create the libs folder if needed
     if sys.platform == 'win32':
         _set_dynlibs(root)
-           
+
     # now we can run bootstrap.py
     python = _python()
 
@@ -403,9 +393,9 @@ def deploy_release(path=None, target=None, archiving='full'):
     _log('Checking binary.')
     buildout_cmd = '%s -c %s -v' % (binary, filename)
     _log('Calling %s' % buildout_cmd)
-    
+
     def runwait(command):
-        p = Popen(command, shell=True, 
+        p = Popen(command, shell=True,
                   stdout=PIPE, stderr=PIPE)
         output = 'x'
         while output:
@@ -414,7 +404,7 @@ def deploy_release(path=None, target=None, archiving='full'):
             sys.stdout.flush()
         errors = p.stderr.read()
         return p.wait(), errors
-        
+
     try:
         exit_code, errors = runwait(buildout_cmd)
         if exit_code != 0:
@@ -434,11 +424,8 @@ def deploy_release(path=None, target=None, archiving='full'):
     try:
         if filename != 'buildout.cfg':
             set_option(buildout, 'buildout', 'extends', filename)
-
-        set_option(buildout, 'buildout', 'install-from-cache',
-                   'true')
+        set_option(buildout, 'buildout', 'install-from-cache', 'true')
         set_option(buildout, 'buildout', 'offline', 'true')
-
         # let's generate the MD5 stamp that will go in the archive
         _log('Generating MD5...')
         open(join(root, 'MD5'), 'w').write(build_md5(root))
@@ -449,11 +436,16 @@ def deploy_release(path=None, target=None, archiving='full'):
         _log('Archiving %s.' % target)
         if archiving == 'none':
             return
-        archive = 'release-%s.tgz' % release_name
-        archive_contents(archive, '.', exclude)
-        _log('%s ok.' % archive)
+        version_txt = join(root, 'version.txt')
+        if os.path.exists(version_txt):
+            version = open(version_txt).read().strip()
+            archive_name = 'release-%s-%s.tgz' % (version, release_name)
+        else:
+            archive_name = 'release-%s.tgz' % release_name
+        archive_contents(archive_name, '.', exclude)
+        _log('%s ok.' % archive_name)
     finally:
-        # setting back the original buildout.cfg file    
+        # setting back the original buildout.cfg file
         f = open(filename, 'w')
         f.write(old_content)
         f.close()
@@ -478,7 +470,7 @@ def copy_archives(from_=None, to=None):
     eggs_target = join(to, 'eggs')
 
     for source, target in ((downloads, downloads_target),
-                           (eggs, eggs_target)):   
+                           (eggs, eggs_target)):
         if not os.path.exists(source):
             continue
         if os.path.exists(target):
@@ -488,7 +480,7 @@ def copy_archives(from_=None, to=None):
                 name = '%s-%d' % (name, i)
                 i += 1
             shutil.move(target, name)
-                
+
         _log('Copying %s to %s' % (source, target))
         os.mkdir(target)
         for root, dirs, filenames in os.walk(source):
@@ -512,7 +504,7 @@ def copy_archives(from_=None, to=None):
 
 def console_build_md5(folder=None):
     _log(build_md5(folder))
-    
+
 def build_md5(folder=None):
     """generate an MD5 stamp, by recursively reading
     the files (just .py, .txt, .cfg, .pt, .zpt,)"""
@@ -520,7 +512,7 @@ def build_md5(folder=None):
     if folder is None:
         if len(sys.argv) > 1:
             folder = sys.argv[1]
-        else:   
+        else:
             sys.exit(1)
     files = {}
     for root, dirs, filenames in os.walk(folder):
@@ -554,7 +546,7 @@ def console_project_eggs(args=None):
         if len(sys.argv) > 1:
             cfg = sys.argv[1]
         if len(sys.argv) > 2:
-            tarball = sys.argv[2] 
+            tarball = sys.argv[2]
         else:
             tarball = None
         if len(sys.argv) > 3:
@@ -568,6 +560,33 @@ def console_project_eggs(args=None):
     _log('\n\nEggs collected:')
     _log('\n'.join(['    - %s' % egg for egg in eggs]))
 
+class EggInstaller(Installer):
+
+    def __init__(self, eggs_directory, links,
+                 index, versions, filter_eggs=None, get_dependencies=False):
+        Installer.__init__(self, dest=eggs_directory, links=links,
+                           index=index, versions=versions)
+        self.get_dependencies = get_dependencies
+        if filter_eggs is None:
+            filter_eggs = []
+        self.filter_eggs = filter_eggs
+        self.tarballs = []
+
+    def _get_dist(self, requirement, ws, always_unzip):
+        dists = Installer._get_dist(self, requirement, ws, always_unzip)
+        for dist in dists:
+            location = dist.location
+            project_name = dist.project_name
+            # not getting dependencies
+            if self.get_dependencies:
+                self.tarballs.append(location)
+            else:
+                for pattern in self.filter_eggs:
+                    if fnmatch(project_name, pattern):
+                        if location not in self.tarballs:
+                            self.tarballs.append(location)
+        return dists
+
 @savewd
 def project_eggs(cfg, tarball=None, filter_eggs=None):
     """Scans for all eggs used by a buildout in eggs, and return their names.
@@ -582,7 +601,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
     def _install(egg):
         if filter_eggs is None:
             return True
-        
+
         for pattern in filter_eggs:
             if fnmatch(egg, pattern):
                 return True
@@ -595,14 +614,14 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
     os.chdir(buildout_dir)
 
     def _(lines):
-        return [l.strip() 
+        return [l.strip()
                 for l in lines.split('\n') if l.strip() != '']
-   
+
     tmp = mkdtemp()
     _log('Working in %s' % tmp)
-    
+
     eggs_directory = join(tmp, 'eggs')
-    os.mkdir(eggs_directory) 
+    os.mkdir(eggs_directory)
 
     install_folder = tmp
     if tarball is None:
@@ -612,14 +631,14 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
     os.mkdir(downloads_directory)
     develop_eggs_directory = join(tmp, 'develop-eggs')
     os.mkdir(develop_eggs_directory)
-    
+
     # forcing a few values
     config = ConfigParser.ConfigParser()
     config.read([cfg])
 
     old_buildout_values = {}
     for name, value in (('eggs-directory', eggs_directory),
-                        ('develop-eggs-directory', develop_eggs_directory), 
+                        ('develop-eggs-directory', develop_eggs_directory),
                         ('download-cache', downloads_directory)):
         if config.has_option('buildout', name):
             old_buildout_values[name] = config.get('buildout', name)
@@ -641,14 +660,14 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
             shutil.copyfile(join(buildout_dir, extend), join(tmp, extend))
             subconfig = ConfigParser.ConfigParser()
             subconfig.read([join(buildout_dir, extend)])
-            _copy_extends(subconfig, buildout_dir, tmp) 
+            _copy_extends(subconfig, buildout_dir, tmp)
 
     _copy_extends(config, buildout_dir, tmp)
 
     buildout = Buildout(cfg_file, [], user_defaults=False)
     buildout._load_extensions()
 
-    # let's get the find-links 
+    # let's get the find-links
     if 'find-links' in buildout['buildout']:
         find_links = _(buildout['buildout']['find-links'])
     else:
@@ -665,7 +684,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
         versions = buildout['buildout']['versions']
         if 'versions' in buildout:
             versions = buildout['versions']
-     
+
     # let's collect the used eggs
     _eggs = []
     for section in buildout:
@@ -676,7 +695,7 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
         if 'recipe' in buildout[section]:
             _eggs.append(buildout[section]['recipe'].strip())
 
-    
+
     eggs = []
     for egg in _eggs:
         egg.strip()
@@ -685,13 +704,14 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
 
     # installing the eggs
     _log('Installing eggs in %s.' % install_folder)
-    installer = Installer(eggs_directory, links=find_links, 
-                          index=index, versions=versions)
+    installer = EggInstaller(eggs_directory, links=find_links,
+                             index=index, versions=versions,
+                             filter_eggs=filter_eggs)
 
     target = join(install_folder, 'eggs')
     if not os.path.exists(target):
         os.mkdir(target)
-    
+
     for egg in eggs:
         if not _install(egg):
             continue
@@ -715,29 +735,27 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
         return egg, egg
 
     # copying the final selection
-    
-    
     def _remove(element):
-        if os.path.isdir(element):  
+        if os.path.isdir(element):
             shutil.rmtree(element, ignore_errors=True)
         else:
             os.remove(element)
-    
+
     def _copy(element, target):
         if os.path.exists(target):
             return
-        if os.path.isdir(element):  
+        if os.path.isdir(element):
             shutil.copytree(element, target)
         else:
             shutil.copyfile(element, target)
-    
+
     if target != eggs_directory:
         for egg in os.listdir(eggs_directory):
             if egg == 'eggs':
                 continue
             eggname = egg.split('-')[0]
             if not _install(eggname):
-                _remove(join(eggs_directory, egg)) 
+                _remove(join(eggs_directory, egg))
                 continue
             if os.path.exists(join(target, egg)):
                 continue
@@ -749,6 +767,21 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
             if not _install(eggname):
                 _remove(join(eggs_directory, egg))
 
+    # let's keep the dists
+    for el in os.listdir(downloads_directory):
+        if el == 'dist':
+            continue
+        _remove(join(downloads_directory, el))
+
+    dist_directory = os.path.join(downloads_directory, 'dist')
+    for tarball_file in os.listdir(dist_directory):
+
+        if tarball_file not in installer.tarballs:
+            _remove(join(dist_directory, tarball_file))
+
+    # we have now a nice downloads_directory
+    _copy(downloads_directory, join(install_folder, 'downloads'))
+
     # let's copy the .cfg files as well
     _log('Copying all cfg files')
     for file_ in os.listdir(buildout_dir):
@@ -769,13 +802,13 @@ def project_eggs(cfg, tarball=None, filter_eggs=None):
         to_ = join(install_folder, file_)
         if os.path.exists(to_):
             os.remove(to_)
-        shutil.copyfile(join(buildout_dir, file_), to_) 
+        shutil.copyfile(join(buildout_dir, file_), to_)
 
     # archiving
     _log('Archiving to %s' % tarball)
-    exclude = ['downloads', 'develop-eggs']    
+    exclude = ['downloads', 'develop-eggs']
     archive_contents(tarball, install_folder, exclude=exclude)
-    installed = ['%s (%s)' % (egg.split('-')[0], egg.split('-')[1]) 
+    installed = ['%s (%s)' % (egg.split('-')[0], egg.split('-')[1])
                  for egg in os.listdir(target)]
     _remove(tmp)
     return installed
@@ -810,12 +843,14 @@ def console_make_svn_structure_for_package(package_directory=None):
     shutil.move(tmp_name, "%s/trunk" % package_directory)
 
     #svn add and ci it, if the parent directory is a working copy
-    try:
-        base.svn_add(package_directory)
-        base.svn_commit('initial import of %s' % package_directory)
-        _log("added %s to svn" % package_directory)
-    except:
-        _log("Current directory is not a working copy. Skip adding %s to svn" % package_directory)
+    if base.has_svn_support():
+        try:
+
+            base.svn_add(package_directory)
+            base.svn_commit('initial import of %s' % package_directory)
+            _log("added %s to svn" % package_directory)
+        except:
+            _log("Current directory is not a working copy. Skip adding %s to svn" % package_directory)
 
     _log('Done...')
 
